import getpass
import elasticsearch
import elasticsearch.helpers

from typing import Dict

class CogStackConn(object):
    def __init__(self, host, port=9200, username: str=None, password: str=None, scheme: str='https',
                 timeout: int=360, max_retries: int=10, retry_on_timeout: bool=True, **kwargs):
        username, password = self._check_auth_details(username, password)

        self.elastic = elasticsearch.Elasticsearch(hosts=[{'host': host, 'port': port}],
                                         http_auth=(username, password),
                                         scheme=scheme,
                                         verify_certs=False,
                                         timeout=timeout,
                                         max_retries=max_retries,
                                         retry_on_timeout=retry_on_timeout,
                                         **kwargs)

    def _check_auth_details(self, username: str=None, password: str=None):
        if username is None:
            username = input("Username:")
        if password is None:
            password = getpass.getpass("Password:")

        # TODO: Implement auth check, for now I assume all is fine
        return username, password


    def get_docs_generator(self, query: Dict, index: str, es_gen_size: int=800, request_timeout: int=840000, **kwargs):
        docs_generator = elasticsearch.helpers.scan(self.elastic,
            query=query,
            index=index,
            size=es_gen_size,
            request_timeout=request_timeout,
            **kwargs)

        return docs_generator


    def bulk_to_cogstack(self):
        # TODO: look the code made for Nazli/Dan
        pass



def tag_skip_and_punct(nlp, name, config):
    r''' Detects and tags spacy tokens that are punctuation and that should be skipped.

     Args:
         nlp (spacy.language.<lng>):
             The base spacy NLP pipeline.
         name (`str`):
             The component instance name.
         config (`medcat.config.Config`):
             Global config for medcat.
    '''
    return _Tagger(nlp, name, config)


# This is not elegant.
tag_skip_and_punct.name = "tag_skip_and_punct"


class _Tagger(object):

    def __init__(self, nlp, name, config):
        self.nlp = nlp
        self.name = name
        self.config = config

    def __call__(self, doc):
        # Make life easier
        cnf_p = self.config.preprocessing

        for token in doc:
            if self.config.punct_checker.match(token.lower_) and token.text not in cnf_p['keep_punct']:
                # There can't be punct in a token if it also has text
                token._.is_punct = True
                token._.to_skip = True
            elif self.config.word_skipper.match(token.lower_):
                # Skip if specific strings
                token._.to_skip = True
            elif cnf_p['skip_stopwords'] and token.is_stop:
                token._.to_skip = True

        return doc



name = 'medcat'

# Hacky patch to the built-in copy module coz otherwise, thinc.config.Config.copy will fail on Python <= 3.6.
# (fixed in python 3.7 https://docs.python.org/3/whatsnew/3.7.html#re)
import sys
if sys.version_info.major == 3 and sys.version_info.minor <= 6:
    import copy
    import re

    copy._deepcopy_dispatch[type(re.compile(''))] = lambda r, _: r



import setuptools
from setuptools.command.install import install
from setuptools.command.develop import develop
from setuptools.command.egg_info import egg_info

with open("./README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="medcat",
    version="1.1.3",
    author="w-is-h",
    author_email="w.kraljevic@gmail.com",
    description="Concept annotation tool for Electronic Health Records",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/CogStack/MedCAT",
    packages=['medcat', 'medcat.utils', 'medcat.preprocessing', 'medcat.cogstack', 'medcat.ner', 'medcat.linking', 'medcat.datasets', 'medcat.deprecated'],
    install_requires=[
        'numpy<1.21.0,>=1.19.0',
        'pandas~=1.0',
        'gensim~=3.8',
        'spacy<3.1.0,>=3.0.1',
        'scipy~=1.5',
        'transformers~=4.5.1',
        'torch~=1.8.1',
        'tqdm<4.50.0,>=4.27',
        'sklearn~=0.0',
        'elasticsearch~=7.10',
        'dill~=0.3.3',
        'datasets~=1.6.0',
        'jsonpickle~=2.0.0',
        ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
